package papi;


/**
 * 2011
 * @author Emilio Francesquini <francesquini@gmail.com> 
 * 
 */
public class Papi4J {
	
	/*
	 * An alternative more Java-like interface for PAPI 
	 * This class is still incomplete, most of the methods 
	 * available can still be found on the PapiJ class
	 */
	
	private static final Papi4J instance = new Papi4J();
	
	public PapiJ pj;
	
	private Papi4J() {	
	}
	
	public static Papi4J getInstance() {
		return instance;		
	}
	
	public void initialize() {
		pj = new PapiJ();
		int retval = pj.library_init(PapiJ.PAPI_VER_CURRENT);
		if (retval < 0)
			fail("Failed Papi Init", retval);
	}
	
	public void shutdown() {
		pj.shutdown();
	}
	
	public EventSet createEventSet() {
		EventSet es = new EventSet();
		doOrCry(pj.create_eventset(es), "Unable to create event set");
		es.pj = pj;
		return es;		
	}
	
	public void setOption (int option, PAPI_option p) {
		doOrCry(pj.set_opt(option, p), "Could not set PAPI option");
	}
	
	public long getRealUsec() {
		return pj.get_real_usec();
	}
	
	public long getRealCycles() {
		return pj.get_real_cyc();
	}
	

	public long getVirtualUsec() {
		return pj.get_virt_usec();
	}
	

	public long getVirtualCycles() {
		return pj.get_virt_cyc();
	}
	
	
	/*========================================*/
	
	
	public static void doOrCry(int retval, String desc) {
		if (retval != PapiJ.PAPI_OK)
			fail(desc, retval);
	}	
	
	public static void fail(String desc, int code) {
		throw new PapiException(desc, code);
	}
	
	public static int getVersion() {
		return PapiJ.PAPI_VER_CURRENT;
	}
	
	public static String getVersionString() {
		int version = getVersion();
		int major =  (version >> 24) & 0xff;
		int minor =  (version >> 16) & 0xff;
		int revision = (version >> 8) & 0xff;
		int increment = version & 0xff;
		return "PAPI v" + major + "." + minor + "." + revision + "." + increment;	
	}
	
}
